#include "batch.h"
#include "border.h"
#include "uimanager.h"
#include "sizedialog.h"
#include "quality.h"
#include "../stub/ifapp.h"
#include "../qimageutils/effectdialogs.h"
#include <qlabel.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qgroupbox.h>
#include <qinputdialog.h>
#include <qlayout.h>
#include <qfontmetrics.h>
#include <qpainter.h>
#include <klocale.h>
#include <kprogress.h>
#include <kapp.h>
#include <kglobal.h>
#include <kconfig.h>
#include <kbuttonbox.h>
#include <kmessagebox.h>
#include <kcolordialog.h>
#include <kimageeffectdebug.h>

void createTextLabel(const QString &text, const QColor &color,
                     const QFont &font, QImage &destImage)
{
    QFontMetrics fm(font);
    QRect r(0, 0, fm.width(text), fm.height());

    // TODO: Get rid of tmp pixmap and call FreeType directly
    QPixmap tmpPix(r.width(), r.height());
    tmpPix.fill(Qt::black);
    QPainter p;
    p.begin(&tmpPix);
    p.setFont(font);
    p.setPen(Qt::white);
    p.drawText(0, 0, r.width(), r.height(), Qt::AlignCenter, text);
    p.end();

    destImage.reset();
    destImage.create(r.width(), r.height(), 32);
    QImage srcImage = tmpPix.convertToImage();
    if(srcImage.depth() < 32)
        srcImage.convertDepth(32);

    unsigned int *dest = (unsigned int *)destImage.bits();
    unsigned int *src = (unsigned int *)srcImage.bits();
    int i, len = r.width()*r.height();

    for(i=0; i < len; ++i){
        if(src[i] == Qt::black.rgb())
            dest[i] = qRgba(0, 0, 0, 0);
        else if(src[i] == Qt::white.rgb())
            dest[i] = color.rgb();
        else{
            // src will be grayscale, so just use red for alpha
            dest[i] = qRgba(color.red(), color.green(), color.blue(),
                            qRed(src[i]));
        }
    }
}

bool applyTextLabel(TextPosition pos, QImage &textImage, QImage &destImage)
{
    if(textImage.width() > destImage.width() ||
       textImage.height() > destImage.height()){
        // TODO: error dlg
        qWarning("Image too small for text label! Skipping.");
        return(false);
    }
    int sx, sy, dx, dy;
    int xstart, ystart;
    unsigned int *src, *dest;
    int r, g, b, alpha;

    if(pos == UpperLeftText){
        xstart = 0;
        ystart = 0;
    }
    else if(pos == BottomLeftText){
        xstart = 0;
        ystart = destImage.height()-textImage.height()-1;
    }
    else if(pos == UpperRightText){
        xstart = destImage.width()-textImage.width()-1;
        ystart = 0;
    }
    else if(pos == BottomRightText){
        xstart = destImage.width()-textImage.width()-1;
        ystart = destImage.height()-textImage.height()-1;
    }
    else{ // center
        xstart = ((destImage.width()-textImage.width())/2)-1;
        ystart = ((destImage.height()-textImage.height())/2)-1;
    }

    for(sy = 0, dy = ystart; sy < textImage.height(); ++sy, ++dy){
        src = (unsigned int *)textImage.scanLine(sy);
        dest = (unsigned int *)destImage.scanLine(dy);
        for(sx = 0, dx = xstart; sx < textImage.width(); ++sx, ++dx){
            alpha = qAlpha(src[sx]);
            if(!alpha)
                ;
            else if(alpha == 255)
                dest[dx] = src[sx];
            else{
                float srcPercent = ((float)alpha)/255.0;
                float destPercent = 1.0-srcPercent;
                r = (int)(qRed(src[sx])*srcPercent + qRed(dest[dx])*destPercent);
                g = (int)(qGreen(src[sx])*srcPercent + qGreen(dest[dx])*destPercent);
                b = (int)(qBlue(src[sx])*srcPercent + qBlue(dest[dx])*destPercent);
                dest[dx] = qRgba(r, g, b, 255);
            }
        }
    }
    return(true);
}

KIFTextDialog::KIFTextDialog(QWidget *parent, const char *name)
    : QDialog(parent, name, true)
{
    KConfig *config = KGlobal::config();
    QString oldGrp = config->group();
    config->setGroup("BatchAddText");

    QVBoxLayout *mainLayout = new QVBoxLayout(this);

    QGroupBox *alignGroup= new QGroupBox(i18n("Text Position"), this);
    alignBtns = new QButtonGroup(this);
    alignBtns->hide();
    QGridLayout *alignLayout = new QGridLayout(alignGroup, 1, 1, 15);

    QRadioButton *ulBtn = new QRadioButton(i18n("Upper Left"), alignGroup);
    alignBtns->insert(ulBtn, UpperLeftText);
    alignLayout->addWidget(ulBtn, 0, 0);
    QRadioButton *urBtn = new QRadioButton(i18n("Upper Right"), alignGroup);
    alignBtns->insert(urBtn, UpperRightText);
    alignLayout->addWidget(urBtn, 0, 2);
    QRadioButton *cBtn = new QRadioButton(i18n("Centered"), alignGroup);
    alignBtns->insert(cBtn, CenterText);
    alignLayout->addWidget(cBtn, 1, 1);
    QRadioButton *llBtn = new QRadioButton(i18n("Lower Left"), alignGroup);
    alignBtns->insert(llBtn, BottomLeftText);
    alignLayout->addWidget(llBtn, 2, 0);
    QRadioButton *lrBtn = new QRadioButton(i18n("Lower Right"), alignGroup);
    alignBtns->insert(lrBtn, BottomRightText);
    alignLayout->addWidget(lrBtn, 2, 2);
    mainLayout->addWidget(alignGroup);
    alignBtns->setButton(0);

    QGroupBox *colorGroup = new QGroupBox(i18n("Text Options"), this);
    QHBoxLayout *colorLayout = new QHBoxLayout(colorGroup, 15);
    QLabel *colorLbl = new QLabel(i18n("Text color:"), colorGroup);
    colorLayout->addWidget(colorLbl);
    colorBtn = new KColorButton(colorGroup);
    colorBtn->setColor(config->readColorEntry("Color", &Qt::black));
    colorLayout->addWidget(colorBtn);
    numInput = new KIntNumInput(100, colorGroup);
    numInput->setLabel(i18n("Opacity"), AlignLeft);
    numInput->setRange(10, 100);
    numInput->setValue(config->readNumEntry("Opacity", 100));
    colorLayout->addWidget(numInput);

    mainLayout->addWidget(colorGroup);

    fontChooser = new KFontChooser(this);
    fontChooser->setSampleText(config->readEntry("Text", i18n("Add your text here!")));
    mainLayout->addWidget(fontChooser);

    mainLayout->addStretch();

    KButtonBox *bbox = new KButtonBox(this);
    bbox->addStretch(1);

    connect(bbox->addButton(i18n("OK")), SIGNAL(clicked()), this,
            SLOT(accept()));
    connect(bbox->addButton(i18n("Cancel")), SIGNAL(clicked()), this,
            SLOT(reject()));
    mainLayout->addWidget(bbox);

    setCaption(i18n("Add Text Options"));
    config->setGroup(oldGrp);
}

void KIFTextDialog::accept()
{
    qWarning("In KIFTextDialog::accept");
    if(result() == Accepted){
        KConfig *config = KGlobal::config();
        QString oldGrp = config->group();
        config->setGroup("BatchAddText");
        config->writeEntry("Color", colorBtn->color());
        config->writeEntry("Opacity", numInput->value());
        config->writeEntry("Text", fontChooser->sampleText());
        config->sync();
    }
    QDialog::accept();
}

TextPosition KIFTextDialog::pos()
{
    if(alignBtns->selected())
        return((TextPosition)alignBtns->id(alignBtns->selected()));
    return(UpperLeftText);
}

bool KIFBatchEffect::effect(EffectType type, KIFThumbView *parent,
                            QStringList &fileList, UIManager *mgr)
{
    QImage img, tmpImg;
    QString fileStr;
    bool saveImg;
    int newW, newH; // used for scale dialog
    TextPosition pos;
    QFileInfo fi;
    int quality; // quality stuff
    bool psudeo, useQuality = false;
    BorderType borderType; // border stuff
    int borderWidth;
    QColor fg, bg;
    double weight;
    double a, f, e;
    bool useColor, ok;
    KImageEffectDebug::NoiseType noiseType;

    // Do we need to do any dialogs for more information before doing effect?

    weight = getWeight(type, ok); // will present a dialog for appropriate effects
    if(!ok)
        return(false);

    if(type == Wave){
        KIFSineWaveDialog dlg;
        if(dlg.exec() != QDialog::Accepted)
            return(false);
        a = dlg.amplitude();
        f = dlg.frequency();
    }
    else if(type == Shade){
        KIFShadeDialog dlg;
        if(dlg.exec() != QDialog::Accepted)
            return(false);
        a = dlg.azimuth();
        e = dlg.elevation();
        useColor = dlg.colorShading();
    }
    else if(type == AddText){
        KIFTextDialog dlg;
        if(dlg.exec() != QDialog::Accepted){
            return(false);
        }
        createTextLabel(dlg.text(), dlg.color(), dlg.font(),
                        tmpImg);
        pos = dlg.pos();

    }
    else if(type == Scale){
        KIFSizeDialog dlg(600, 500);
        int result = dlg.exec();
        if(result == QDialog::Accepted){
            newW = dlg.resultWidth();
            newH = dlg.resultHeight();
        }
        if(result == QDialog::Rejected)
            return(false);
    }
    else if(type == Border){
        KIFBorderDialog dlg;
        if(dlg.exec() == QDialog::Accepted){
            borderType = dlg.borderType();
            borderWidth = dlg.borderWidth();
            fg = dlg.fgColor();
            bg = dlg.bgColor();
        }
        else
            return(false);
    }
    else if(type == AddNoise){
        QStringList typeList;
        typeList.append(i18n("Uniform"));
        typeList.append(i18n("Gaussian"));
        typeList.append(i18n("Multiplicative Gaussian"));
        typeList.append(i18n("Impulse"));
        typeList.append(i18n("Laplacian"));
        typeList.append(i18n("Poisson"));
        QString type = QInputDialog::getItem(i18n("Noise Type"),
                                       i18n("Please select a noise type."),
                                       typeList, 0, false, &ok);
        if(!ok)
            return(false);
        if(type == i18n("Uniform"))
            noiseType = KImageEffectDebug::UniformNoise;
        else if(type == i18n("Gaussian"))
            noiseType = KImageEffectDebug::GaussianNoise;
        else if(type == i18n("Multiplicative Gaussian"))
            noiseType = KImageEffectDebug::MultiplicativeGaussianNoise;
        else if(type == i18n("Impulse"))
            noiseType = KImageEffectDebug::ImpulseNoise;
        else if(type == i18n("Laplacian"))
            noiseType = KImageEffectDebug::LaplacianNoise;
        else if(type == i18n("Poisson"))
            noiseType = KImageEffectDebug::PoissonNoise;
    }
    if(type == Implode || type == Swirl || type == Wave){
        KColorDialog dlg(NULL, NULL, true);
        dlg.setCaption(i18n("Select a Background Color"));
        dlg.setColor(Qt::white);
        if(dlg.exec() != QDialog::Accepted)
            return(false);
        bg = dlg.color();
    }

    // look to see if any of the files are in PNG or JPG format
    for(QStringList::Iterator it = fileList.begin(); it != fileList.end(); ++it){
        fileStr = parent->currentDir + "/" + (*it);
        if(kifImageType(fileStr) == "JPEG" || kifImageType(fileStr) == "PNG"){
            KIFQualityDialog dlg(true, "");
            if(dlg.exec() == QDialog::Rejected){
                return(false);
            }
            quality = dlg.quality();
            psudeo = dlg.psudeoClass();
            useQuality = true;
            break;
        }
    }
    // apply effect
    for(QStringList::Iterator it = fileList.begin(); it != fileList.end(); ++it){
        fileStr = parent->currentDir + "/" + (*it);
        if(!kifLoadImage(img, fileStr)){
            if(KMessageBox::warningContinueCancel(parent,
                                                  i18n("Cannot load file: ") + (*it),
                                                  i18n("Pixie Batch Effect Error!"),
                                                  i18n("Continue"),
                                                  QString::null, true) ==
               KMessageBox::Cancel){
                mgr->statusLbl->setText(i18n("Cancelled!"));
                if(it != fileList.begin()) // some files modified
                    return(true);
                else if(fileList.count() == 1)
                    return(false); // this was only item
            }
        }
        //if(img.depth() < 32)
        //    img = img.convertDepth(32);
        saveImg = true;
        switch(type){
        case AddText:
            if(img.depth() < 32)
                img = img.convertDepth(32);
            mgr->statusLbl->setText(i18n("Adding text to ") + (*it) + i18n("..."));
            kapp->processEvents();
            saveImg = applyTextLabel(pos, tmpImg, img);
            break;
        case Scale:
            if(img.width() > newW || img.height() > newH){
                mgr->statusLbl->setText(i18n("Scaling ") + (*it) + i18n("..."));
                kapp->processEvents();
                if(img.width() > img.height()){
                    float percent = (((float)newW)/img.width());
                    int h = (int)(img.height()*percent);
                    img = img.smoothScale(newW, h);
                }
                else{
                    float percent = (((float)newH)/img.height());
                    int w = (int)(img.width()*percent);
                    img = img.smoothScale(w, newH);
                }
            }
            else
                saveImg = false;
            break;
        case Border:
            if(img.depth() < 32)
                img = img.convertDepth(32);
            mgr->statusLbl->setText(i18n("Adding border to ") + (*it) + i18n("..."));
            kapp->processEvents();
            if(borderType == Solid){
                KIFBorderEffect::solid(img, tmpImg, fg, borderWidth);
            }
            else if(borderType == Bevel){
                KIFBorderEffect::bevel(img, tmpImg, fg, bg, borderWidth);
            }
            else if(borderType == Liquid){
                KIFBorderEffect::liquid(img, tmpImg, fg, bg, borderWidth);
            }
            else if(borderType == RoundCorner){
                KIFBorderEffect::roundCorner(img, tmpImg, bg);
            }
            img = tmpImg;
            break;
        case Grayscale:
            mgr->statusLbl->setText(i18n("Grayscaling ") + (*it) + i18n("..."));
            kapp->processEvents();
            KImageEffectDebug::toGray(img);
            break;
        case Invert:
            mgr->statusLbl->setText(i18n("Inverting ") + (*it) + i18n("..."));
            kapp->processEvents();
            img.invertPixels(false);
            break;
        case FlipH:
            mgr->statusLbl->setText(i18n("Flipping ") + (*it) + i18n("..."));
            kapp->processEvents();
            img = img.mirror(true, false);
            break;
        case FlipV:
            mgr->statusLbl->setText(i18n("Flipping ") + (*it) + i18n("..."));
            kapp->processEvents();
            img = img.mirror(false, true);
            break;
        case Rot90:
            img = KImageEffectDebug::rotate(img, KImageEffectDebug::Rotate90);
            break;
        case Rot180:
            img = KImageEffectDebug::rotate(img, KImageEffectDebug::Rotate180);
            break;
        case Rot270:
            img = KImageEffectDebug::rotate(img, KImageEffectDebug::Rotate270);
            break;
        case Blur:
            img = KImageEffectDebug::blur(img, weight);
            break;
        case Normalize:
            KImageEffectDebug::normalize(img);
            break;
        case Equalize:
            KImageEffectDebug::equalize(img);
            break;
        case Despeckle:
            img = KImageEffectDebug::despeckle(img);
            break;
        case AddNoise:
            // TODO dialog options
            img = KImageEffectDebug::addNoise(img, noiseType);
            break;
        case Edge:
            img = KImageEffectDebug::edge(img, weight);
            break;
        case Implode:
            img = KImageEffectDebug::implode(img, weight, bg.rgb());
            break;
        case OilPaint:
            img = KImageEffectDebug::oilPaint(img, (int)weight);
            break;
        case Sharpen:
            img = KImageEffectDebug::sharpen(img, weight);
            break;
        case Spread:
            img = KImageEffectDebug::spread(img, (int)weight);
            break;
        case Swirl:
            img = KImageEffectDebug::swirl(img, weight, bg.rgb());
            break;
        case Shade:
            img = KImageEffectDebug::shade(img, useColor, a, e);
            break;
        case Solarize:
            KImageEffectDebug::solarize(img, weight);
            break;
        case Wave:
            img = KImageEffectDebug::wave(img, a, f, bg.rgb());
            break;
        case Emboss:
            img = KImageEffectDebug::emboss(img);
            break;
        case IncContrast:
            KImageEffectDebug::contrastHSV(img, true);
            break;
        case DecContrast:
            KImageEffectDebug::contrastHSV(img, false);
            break;
        case Charcoal:
            img = KImageEffectDebug::charcoal(img);
            break;
        case Threshold:
            KImageEffectDebug::threshold(img, (int)weight);
            break;
        default:
            break;
        }
        if(saveImg){
            mgr->statusLbl->setText(i18n("Saving ") + (*it) + i18n("..."));
            kapp->processEvents();
            QString format = kifImageType(fileStr);
            bool result;
            if(format == "JPEG" || format == "PNG"){
                if(psudeo && img.depth() > 8){
                    QImage newImg = img.convertDepth(8, QPixmap::ColorOnly |
                                                     QPixmap::DiffuseDither |
                                                     QPixmap::ThresholdAlphaDither |
                                                     QPixmap::AvoidDither);
                    if(format == "JPEG")
                        result = kifSaveImage(newImg, fileStr, format.ascii(), quality*10);
                    else
                        result = kifSaveImage(newImg, fileStr, format.ascii(), quality);
                }
                else{
                    if(format == "JPEG")
                        result = kifSaveImage(img, fileStr, format.ascii(), quality*10);
                    else
                        result = kifSaveImage(img, fileStr, format.ascii(), quality);
                }
            }
            else
                result = kifSaveImage(img, fileStr, format.ascii());

            if(!result){
                if(KMessageBox::warningContinueCancel(parent,
                                                      i18n("Cannot save file: ") + (*it),
                                                      i18n("Pixie Batch Effect Error!"),
                                                      i18n("Continue"),
                                                      QString::null, true) ==
                   KMessageBox::Cancel){
                    mgr->statusLbl->setText(i18n("Cancelled!"));
                    if(it != fileList.begin()) // some files modified
                        return(true);
                    else if(fileList.count() == 1)
                        return(false); // this was only item
                }
            }
        }
    }
    mgr->statusLbl->setText(i18n("Done!"));
    return(true);
}

double KIFBatchEffect::getWeight(EffectType type, bool &ok)
{
    // Do addnoise
    if(type == Threshold)
        return(QInputDialog::getInteger(i18n("Threshold"),
                                        i18n("Please threshold value(0-255)"),
                                        128, 0, 255, 1, &ok));
    else if(type == Solarize)
        return(QInputDialog::getDouble(i18n("Solarize Extent"),
                                       i18n("Please enter the extent of solarization(0 - 99.9)"),
                                       50, 0, 99.9, 1, &ok));
    else if(type == Charcoal)
        return(QInputDialog::getDouble(i18n("Charcoal Factor"),
                                       i18n("Please enter the factor for detecting lines(0 - 99.9)"),
                                       50, 0, 99.9, 1, &ok));
    else if(type == Blur)
        return(QInputDialog::getDouble(i18n("Blur"),
                                       i18n("Please enter the amount to blur(0 - 99.9)"),
                                       90, 0, 99.9, 1, &ok));
    else if(type == Edge)
        return(QInputDialog::getDouble(i18n("Edge Detect"),
                                       i18n("Please enter the factor for detecting lines(0 - 99.9)"),
                                       50, 0, 99.9, 1, &ok));
    else if(type == Implode)
        return(QInputDialog::getDouble(i18n("Implode"),
                                       i18n("Please enter the extent of implosion(0 - 99.9)"),
                                       30, 0, 99.9, 1, &ok));
    else if(type == OilPaint)
        return(QInputDialog::getInteger(i18n("Oil Paint"),
                                        i18n("Please enter the radius of the brush(0-20)"),
                                        3, 0, 20, 1, &ok));
    else if(type == Sharpen)
        return(QInputDialog::getDouble(i18n("Sharpen"),
                                       i18n("Please enter the amount to sharpen(0 - 99.9)"),
                                       30, 0, 99.9, 1, &ok));
    else if(type == Spread)
        return(QInputDialog::getInteger(i18n("Spread"),
                                        i18n("Please enter the radius of the brush(0-20)"),
                                        3, 0, 20, 1, &ok));
    else if(type == Swirl)
        return(QInputDialog::getDouble(i18n("Swirl"),
                                       i18n("Please enter the tightness of the swirl(0 - 99.9)"),
                                       50, 0, 99.9, 1, &ok));
    ok = true;
    return(0.0);
}


