#include "dirtree.h"
#include "fileop.h"

#include <klocale.h>
#include <qdir.h>
#include <qfile.h>
#include <qdragobject.h>
#include <qpopupmenu.h>
#include <qheader.h>

#include <kiconloader.h>
#include <kglobal.h>
#include <kmimetype.h>

PixieBaseItem::PixieBaseItem(PixieDirItem *parent, const QString &file)
    : QListViewItem(parent)
{
    parentDir = parent;
    fileStr = file;
}

PixieBaseItem::PixieBaseItem(QListView *parent, const QString &file)
    : QListViewItem(parent)
{
    parentDir = NULL;
    fileStr = file;

}

bool PixieBaseItem::isDir()
{
    return(dirFlag);
}


QString PixieBaseItem::text(int) const
{
    return(fileStr);
}

QString PixieBaseItem::fullPath()
{
    QString str;
    if (parentDir){
        str = parentDir->fullPath() + fileStr;
        if(dirFlag)
            str += "/";
    }
    else
	str = fileStr;
    return(str);
}

PixieFileItem::PixieFileItem(PixieDirItem *parent, const QString &file)
    : PixieBaseItem(parent, file)
{
    dirFlag = false;
    setPixmap(0, KMimeType::pixmapForURL(KURL("file:"+fullPath()),
                                         0, KIcon::Small, KIcon::SizeSmall));
}


PixieDirItem::PixieDirItem(PixieDirItem *parent, const QString &file)
    : PixieBaseItem(parent, file)
{
    dirFlag = true;
    QFileInfo fi(fullPath());
    hasAccess = fi.isExecutable();
    if(hasAccess)
        setPixmap(0, SmallIcon("folder"));
    else{
        setExpandable(false);
        setPixmap(0, SmallIcon("folder_locked"));
    }
}



PixieDirItem::PixieDirItem(QListView *parent, const QString &file)
    : PixieBaseItem(parent, file)
{
    dirFlag = true;
    QFileInfo fi(fullPath());
    hasAccess = fi.isExecutable();
    if(hasAccess)
        setPixmap(0, SmallIcon("folder"));
    else{
        setExpandable(false);
        setPixmap(0, SmallIcon("folder_locked"));
    }
}

void PixieDirItem::setOpen(bool open)
{
    if(!hasAccess){
        qWarning("Open called for locked dir!");
        QListViewItem::setOpen(false);
        return;
    }

    PixieDirTree *tree = (PixieDirTree *)listView();
    if(open)
        setPixmap(0, SmallIcon("folder_open"));
    else{
        if(hasAccess)
            setPixmap(0, SmallIcon("folder"));
        else
            setPixmap(0, SmallIcon("folder_locked"));
    }
    if(open && !childCount()){
        QString pathStr(fullPath());
        QDir currentDir(pathStr);
        if(tree->isDirOnly())
            currentDir.setFilter(QDir::Dirs);
        else
            currentDir.setFilter(QDir::All);
        currentDir.setSorting(QDir::Name | QDir::DirsFirst | QDir::Reversed);
        if(!currentDir.isReadable()){
            hasAccess = false;
            setExpandable(false);
            return;
        }
        listView()->setUpdatesEnabled(false);
        const QFileInfoList * fileList = currentDir.entryInfoList();
        if(fileList){
            QFileInfoListIterator it(*fileList);
            QFileInfo *fi;
            for(; (fi=it.current()); ++it){
                if (fi->fileName() == "." || fi->fileName() == "..")
                    ;
                else{
                    if(fi->isDir())
                        (void)new PixieDirItem(this, fi->fileName());
                    else if(!tree->isDirOnly())
                        (void)new PixieFileItem(this, fi->fileName());
                }
            }
        }
	listView()->setUpdatesEnabled( TRUE );
    }
    QListViewItem::setOpen(open);
}


PixieDirTree::PixieDirTree(bool dirOnly, QWidget *parent, const char *name)
    : QListView(parent, name)
{
    prevSelectedItem = NULL;
    //dirWatch = new KDirWatch(3000);
    noFiles = dirOnly;
    setAcceptDrops(true);
    //setItemsRenameable(true);

    setSorting(-1);
    //connect(dirWatch, SIGNAL(dirty(const QString &)), this,
    //        SLOT(slotDirChanged(const QString &)));
    //connect(this, SIGNAL(dropped(QDropEvent *, QListViewItem *)), this,
    //        SLOT(slotDropped(QDropEvent *, QListViewItem *)));
    connect(this, SIGNAL(doubleClicked(QListViewItem *)), this,
            SLOT(slotClicked(QListViewItem *)));
    connect(this, SIGNAL(returnPressed(QListViewItem *)), this,
            SLOT(slotClicked(QListViewItem *)));
    connect(this, SIGNAL(expanded(QListViewItem *)), this,
            SLOT(slotDirOpened(QListViewItem *)));
    connect(this, SIGNAL(collapsed(QListViewItem *)), this,
            SLOT(slotDirClosed(QListViewItem *)));
    connect(this, SIGNAL(rightButtonClicked(QListViewItem *, const QPoint &, int)), this,
            SLOT(slotRightClick(QListViewItem *, const QPoint &, int)));
    //connect(this, SIGNAL(itemRenamed(QListViewItem*, const QString &, int)), this,
    //        SLOT(slotRename(QListViewItem *, const QString &, int)));

    //dirWatch->startScan();
    addColumn(i18n("Directory Tree"));
    setTreeStepSize(20);

    PixieDirItem *item = new PixieDirItem(this, "/");
    item->setOpen(true);
}

PixieDirTree::~PixieDirTree()
{
    //delete dirWatch;
}

void PixieDirTree::slotClicked(QListViewItem *i)
{
    PixieBaseItem *item = (PixieBaseItem *)i;
    if(item->isDir()){
        item = (PixieDirItem *)i;
        item->setOpen(true);
        item->repaint();
        qWarning("Clicked folder %s", item->fullPath().latin1());
    }
}

void PixieDirTree::setPath(const QString &path)
{
    QListViewItemIterator it( this );
    for ( ++it; it.current(); ++it ) {
	it.current()->setOpen( FALSE );
    }

    qWarning("In setPath for %s", path.latin1());
    QStringList lst(QStringList::split( "/", path ));
    QListViewItem *item = firstChild();
    QStringList::Iterator it2 = lst.begin();
    PixieBaseItem *childItem = NULL;

    for( ; it2 != lst.end(); ++it2 ) {
        qWarning("Looking for %s",  (*it2).latin1());
        item->setOpen(true);
        childItem = (PixieBaseItem*)item->firstChild();
        item = NULL;
        while(childItem){
            if(childItem->isDir() && childItem->text(0) == *it2){
                qWarning("Found match for %s", (*it2).latin1());
                childItem->setOpen(true);
                item = childItem;
                break;
            }
            else
                childItem = (PixieBaseItem*)childItem->itemBelow();
        }
        if(!item)
            return;
    }
    setCurrentItem(item);
    setSelected(item, true);
    ensureItemVisible(item);
}

void PixieDirTree::slotDirChanged(const QString &path)
{
    qWarning("slotDirChanged");
    QStringList lst(QStringList::split( "/", path ));
    QListViewItem *item = firstChild();
    QStringList::Iterator it2 = lst.begin();
    PixieBaseItem *childItem = NULL;
    bool open = isOpen(item);

    for( ; it2 != lst.end(); ++it2 ) {
        qWarning("Looking for %s",  (*it2).latin1());
        childItem = (PixieBaseItem*)item->firstChild();
        item = NULL;
        while(childItem){
            if(childItem->isDir() && childItem->text(0) == *it2){
                item = childItem;
                break;
            }
            else
                childItem = (PixieBaseItem*)childItem->itemBelow();
        }
        if(!item)
            return;
    }
    QListViewItem *tmp;
    childItem = (PixieBaseItem*)item->firstChild();
    setUpdatesEnabled(false);
    while(childItem){
        tmp = childItem;
        childItem = (PixieBaseItem*)childItem->nextSibling();
        delete tmp;
    }
    if(open){
        // this is essentially the same as in setOpen, but just calling
        // that instead is buggy right now so we do it here
        QDir currentDir(path);
        if(noFiles)
            currentDir.setFilter(QDir::Dirs);
        else
            currentDir.setFilter(QDir::All);
        currentDir.setSorting(QDir::Name | QDir::DirsFirst | QDir::Reversed);
        const QFileInfoList * fileList = currentDir.entryInfoList();
        if(fileList){
            QFileInfoListIterator it(*fileList);
            QFileInfo *fi;
            for(; (fi=it.current()); ++it){
                if (fi->fileName() == "." || fi->fileName() == "..")
                    ;
                else{
                    if(fi->isDir())
                        (void)new PixieDirItem((PixieDirItem*)item, fi->fileName());
                    else if(!noFiles)
                        (void)new PixieFileItem((PixieDirItem*)item, fi->fileName());
                }
            }
        }
    }
    setUpdatesEnabled(true);
}

void PixieDirTree::slotDirOpened(QListViewItem *i)
{
    //dirWatch->addDir(((PixieDirItem*)i)->fullPath());
}

void PixieDirTree::slotDirClosed(QListViewItem *i)
{
    //dirWatch->removeDir(((PixieDirItem*)i)->fullPath());
}


void PixieDirTree::slotRightClick(QListViewItem *, const QPoint &, int)
{

}

void PixieDirTree::dragEnterEvent(QDragEnterEvent *ev)
{
    PixieBaseItem *item = (PixieBaseItem *)itemAt(ev->pos());
    if(!item)
        return;
    qWarning("DragEnterEvent on %s", item->text(0).latin1());
    dndItem = item;
    prevSelectedItem = selectedItem();
    setCurrentItem(item);
    if(item->isDir() && ((PixieDirItem*)item)->canOpen() &&
       QUriDrag::canDecode(ev)){
        ev->accept();
    }
    else
        ev->ignore();
}

void PixieDirTree::dragLeaveEvent(QDragLeaveEvent *)
{
    dndItem = NULL;
    if(prevSelectedItem)
        setCurrentItem(prevSelectedItem);

}

void PixieDirTree::dragMoveEvent(QDragMoveEvent *ev)
{
    PixieBaseItem *item = (PixieBaseItem *)itemAt(ev->pos());
    if(!item)
        return;
    if(item != dndItem){
        qWarning("DragMoveEvent on %s", item->text(0).latin1());
        dndItem = item;
        setCurrentItem(item);
    }
    if(item->isDir() && ((PixieDirItem*)item)->canOpen() &&
       QUriDrag::canDecode(ev)){
        ev->accept();
    }
    else
        ev->ignore();

}


void PixieDirTree::dropEvent(QDropEvent *ev)
{

    PixieBaseItem *item = (PixieBaseItem *)itemAt(ev->pos());

    qWarning("Drop on %s", item->fullPath().latin1());
    if(!item->isDir() || !((PixieDirItem*)item)->canOpen()){
        qWarning("Don't have perms to drop here!");
        return;
    }

    QStringList fileList;
    if(!QUriDrag::decodeLocalFiles(ev, fileList)){
        qWarning("Pixie: Can't decode drop.");
        return;
    }
    if(!fileList.count())
        return;
    int op;
    QPopupMenu opPopup;
    opPopup.insertItem(i18n("&Copy Here"), 1);
    opPopup.insertItem(i18n("&Move Here"), 2);
    opPopup.insertItem(i18n("&Link Here"), 3);

    //QPoint pos = contentsToViewport(ev->pos());
    op = opPopup.exec(mapToGlobal(ev->pos()));
    switch(op){
    case 1:
        ev->setAction(QDropEvent::Copy);
        break;
    case 2:
        ev->setAction(QDropEvent::Move);
        break;
    case 3:
        ev->setAction(QDropEvent::Link);
        break;
    default:
        return;
    }
    // TESTING!!
    KIFFileTransfer::transferFiles(fileList, item->fullPath(), ev->action());
}






